//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : SPI_ctrl.c
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------


// Include the board file description
#include "Board.h"
#include "eepromhandler.h"
#include "SPI_ctrl.h"

extern void APPLI (void);

extern char  message[100];


//CKGR variables

#define PMC_TEST_DONE 			1
#define CKGR_TEST_DONE   		1


#define DISPLAYOFF  			0
#define FIRSTPAGE   			1
#define MAINMENU    			2
#define PMC_MENU    			4
#define CKGR_MENU   			5

#define OFF 							0

#define TC0  							0
#define TC1  							1
#define TC2  							2

// Globals
unsigned long volatile spi_Status;      // Image of spi status register

unsigned long volatile Number_Of_Error;

unsigned char volatile spi_Rdrf_Flag;		// spi Recieve Data Register Full  flag
unsigned char volatile spi_Tdre_Flag;		// spi Transmit Data Register Empty flag

unsigned long volatile spi_Txempty_Flag;    // spi Transmission Register Empty flag

unsigned char volatile spi_Modf_Flag;		// spi Mode Fault Error flag
unsigned char volatile spi_Ovred_Flag;		// spi Overrun Error flag
unsigned char volatile spi_Nssr_Flag;		// spi NSS rising flag

unsigned char volatile spi_Spiens_Flag;		// spi Enable Register flag
unsigned long volatile spi_Endrx_Flag;
unsigned long volatile spi_Endtx_Flag;
unsigned long volatile spi_Rxbuff_Flag;
unsigned long volatile spi_Txbuff_Flag;

//*--------------------------------------------------------------------------------------
//* Function Name       	: WaitUntilReady4TX
//* Object              	: Wait until ready for transmission
//* Input Parameters    : none. 
//* Output Parameters  : none
//*--------------------------------------------------------------------------------------

int WaitUntilReady4TX ( AT91PS_SPI pSPI )
{
  	int						Data_Recieved = 0;
  	ULONG					WaitLoop = 0xFFFF;
  	
  spi_Status = pSPI -> SPI_SR;
	while (( spi_Status & AT91C_SPI_TXBUFE ) == 0 )
	{	
 		spi_Status = pSPI -> SPI_SR;
 		WaitLoop--;
 		if ( WaitLoop == 0 )
 			return -1;
	};		
	if (( spi_Status & AT91C_SPI_RXBUFF ) > 0 )
		Data_Recieved = (pSPI -> SPI_RDR & 0x000FFFFF); // Read RDR and mask non used bits
	 		
 		return Data_Recieved;
};
 
//*--------------------------------------------------------------------------------------
//* Function Name       	: WaitUntilReady4TX
//* Object              	: Wait until ready for transmission
//* Input Parameters    : none. 
//* Output Parameters  : none
//*--------------------------------------------------------------------------------------

int WaitUntilReady2RX ( AT91PS_SPI pSPI )
{
  int						Data_Recieved = 0;
  ULONG					WaitLoop = 0xFFFF;
  
  spi_Status = pSPI -> SPI_SR;
	while (( spi_Status & AT91C_SPI_RXBUFF ) == 0 )	
	{	
 		spi_Status = pSPI -> SPI_SR;
 		WaitLoop--;
 		if ( WaitLoop == 0 )
 			return ( 1 << 31 );
	};		
	Data_Recieved = (pSPI -> SPI_RDR & 0x000FFFFF); // Read RDR and mask non used bits
	 		
 	return Data_Recieved;
};
 
//*--------------------------------------------------------------------------------------
//* Function Name       : Wait
//* Object              : Software waiting loop
//* Input Parameters    : Speed
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
void Wait (unsigned long Speed)
{
    unsigned long waiting_time;
    
    for(waiting_time = 0; waiting_time < Speed; waiting_time++);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void AT91F_SPI_Interrupt_handler (void)
{
  	AT91PS_SPI     pSPI = AT91C_BASE_SPI0;

  	spi_Status = pSPI -> SPI_SR;

	// A data has been recieved and transmitted to the RBR.
	if (spi_Status & AT91C_SPI_RDRF)	
	{
		spi_Rdrf_Flag = 1;
		pSPI -> SPI_IDR = AT91C_SPI_RDRF; // Interrupt is disabled to avoid inopportune interrupt 
	}

	// The last data written in the TDR has been transmitted to the serializer
	if (spi_Status & AT91C_SPI_TDRE)	
	{
		spi_Tdre_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_TDRE; // Interrupt is disabled to avoid inopportune interrupt 
	}

// A mode fault has occured (NCS0 was tied low).
	if (spi_Status & AT91C_SPI_MODF)	
	{
		spi_Modf_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_MODF; // Interrupt is disabled to avoid inopportune interrupt 
	}

	// An overrun error has occured.
	if (spi_Status & AT91C_SPI_OVRES)	
	{
		spi_Ovred_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_OVRES; // Interrupt is disabled to avoid inopportune interrupt 
	}
	
	// The Receive Counter Register has reached 0 since the last write in SPI_RCR or SPI_RNCR.
	if (spi_Status & AT91C_SPI_ENDRX)	
	{
		spi_Endrx_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_ENDRX; // Interrupt is disabled to avoid inopportune interrupt 
	}

	// The Transmit Counter Register has reached 0 since the last write in SPI_TCR or SPI_TNCR.
	if (spi_Status & AT91C_SPI_ENDTX)	
	{
		spi_Endtx_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_ENDTX; // Interrupt is disabled to avoid inopportune interrupt 
	}	

	// Both SPI_RCR and SPI_RNCR have a value of 0.
	if (spi_Status & AT91C_SPI_RXBUFF)	
	{
		spi_Rxbuff_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_RXBUFF; // Interrupt is disabled to avoid inopportune interrupt 
	}	

	// Both SPI_TCR and SPI_TNCR have a value of 0.
	if (spi_Status & AT91C_SPI_TXBUFE)	
	{
		spi_Txbuff_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_TXBUFE; // Interrupt is disabled to avoid inopportune interrupt 
	}

	// NSS has risen
	if (spi_Status & AT91C_SPI_NSSR)	
	{
		spi_Nssr_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_NSSR; // Interrupt is disabled to avoid inopportune interrupt 
	}

	// TDR and shift registers are empty
	if (spi_Status & AT91C_SPI_TXEMPTY)	
	{
		spi_Txempty_Flag  = 1;
		pSPI -> SPI_IDR = AT91C_SPI_TXEMPTY; // Interrupt is disabled to avoid inopportune interrupt 
	}
}

//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_SWReset
//* Object              	: SPI Software reset and check reg after reset
//* Input Parameters    : none.
//* Output Parameters  : none.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_SWReset (const AT91PS_SPI pSPI)
{
	pSPI -> SPI_CR  = AT91C_SPI_SWRST; // SW reset
}



//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_En_Dis
//* Object              	: SPI Enable  Disable
//* Input Parameters    : Enable.
//* Output Parameters  : error.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_En_Dis (const AT91PS_SPI pSPI, unsigned char Enable)
{
	volatile unsigned int error;

	error = 0;

	switch (Enable)
	{ 
		case ENABLE_SPI:
		pSPI -> SPI_CR  = AT91C_SPI_SPIEN; // SPI is enable
		Wait (0xFF);
		break;

		case DISABLE_SPI:
		pSPI -> SPI_CR  = AT91C_SPI_SPIDIS; // SPI is Disable
		Wait (0xFF);
		break;

		case ENABLE_DISABLE_SPI:
		pSPI -> SPI_CR  = (AT91C_SPI_SPIDIS | AT91C_SPI_SPIEN); // Enable and disable are set @ the same time => SPI shoulb be disable
		Wait (0xFF);
		break;
	}
}


//*--------------------------------------------------------------------------------------
//* Function Name       : AT91F_Set_SPI_CSR
//* Object              : Allows programming SPI_CSR registers
//* Input Parameters    : Csr_Value.
//* Input Parameters    : Csr_Number.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void AT91F_Set_SPI_CSR (const AT91PS_SPI pSPI, unsigned long Csr_Value, unsigned long Csr_Number)
{
	switch (Csr_Number)
	{
		case CSR0:
		pSPI -> SPI_CSR[CSR0]  = Csr_Value; // CSR0 <- Csr_Value
		break;

		case CSR1:
		pSPI -> SPI_CSR[CSR1]  = Csr_Value; // CSR1 <- Csr_Value
		break;
		
		case CSR2:
		pSPI -> SPI_CSR[CSR2]  = Csr_Value; // CSR2 <- Csr_Value
		break;

		case CSR3:
		pSPI -> SPI_CSR[CSR3]  = Csr_Value; // CSR3 <- Csr_Value
		break;

		default:
		break; 
	};
};

//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_Read_Data
//* Object              	: Read SPI_RDR
//* Input Parameters    : none.
//* Output Parameters  : Data_Recieved.
//*--------------------------------------------------------------------------------------
unsigned long AT91F_SPI_Read_Data (const AT91PS_SPI pSPI)
{
	unsigned long Data_Recieved;

	spi_Rdrf_Flag = 0;
	pSPI -> SPI_IER = AT91C_SPI_RDRF; // Disabled in the interrupt handler

	while (spi_Rdrf_Flag == 0 );

	Data_Recieved = (pSPI -> SPI_RDR & 0x000FFFFF); // Read RDR and mask non used bits

	pSPI -> SPI_IER = AT91C_SPI_RDRF; // RDF re enabled (see handler)

	return Data_Recieved; // Return RDR (Read Data + PCS)
}


//*--------------------------------------------------------------------------------------
//* Function Name       : AT91F_SPI_Send_Data
//* Object              : Send a data
//* Input Parameters    : Data_To_Send.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_Send_Data (const AT91PS_SPI pSPI, unsigned long Data_To_Send )
{
	// IMPORTANT: Data_To_Send = TD (in fixed mode)
	// IMPORTANT: Data_To_Send = TD + PCS + LASTXFER (in variable mode)
  spi_Tdre_Flag = 0;

	Data_To_Send &= 0x010FFFFF; // mask non used bits
	pSPI -> SPI_TDR = Data_To_Send;
}


//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_Open
//* Object              	: Open the SPI0
//* Input Parameters    : none.
//* Output Parameters  : error.
//*--------------------------------------------------------------------------------------
void  AT91F_SPI_Open ( AT91PS_SPI pSPI  )
{
// MISO, MOSI, CLK, NPCS[3:0] are configured
// SPI (IP) clock is activated
// Interrupt Handler is configured
// SPI is resetted

		// Reset the SPI (note it is possible to reset the SPI even if SPI is disabled)
		AT91F_SPI_SWReset (AT91C_BASE_SPI0);

		spi_Txempty_Flag = 0;

				pSPI -> SPI_IER =	( AT91C_SPI_RDRF + 
														AT91C_SPI_TDRE + 
														AT91C_SPI_MODF + 
														AT91C_SPI_OVRES+ 
														AT91C_SPI_NSSR + 
														AT91C_SPI_TXEMPTY + 
														AT91C_SPI_SPIENS
														);

		 AT91F_SPI_En_Dis ( pSPI, ENABLE_SPI);
		spi_Txempty_Flag = 0;

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Spi_Master ( AT91PS_SPI pSPI )
{
	
	spi_Modf_Flag = 0;
	spi_Ovred_Flag = 0;
	spi_Nssr_Flag = 0;
	spi_Txempty_Flag = 0;
	spi_Spiens_Flag = 0;
	spi_Endrx_Flag = 0;
	spi_Endtx_Flag = 0;
	spi_Rxbuff_Flag = 0;
	spi_Txbuff_Flag = 0;
	spi_Status = 0;
	AT91F_SPI_Open ( pSPI );	

//888888  pSPI -> SPI_IER =  	(AT91C_SPI_RDRF + AT91C_SPI_TDRE ); //+ AT91C_SPI_MODF + AT91C_SPI_OVRES+ 
//		 AT91C_SPI_NSSR + AT91C_SPI_TXEMPTY + AT91C_SPI_SPIENS);

	// Cfg SPI MR
//		pSPI -> SPI_MR = (AT91C_SPI_MSTR + AT91C_SPI_MODFDIS + AT91C_SPI_PS + AT91C_SPI_PCSDEC ); //Variable addresses
		pSPI -> SPI_MR = (AT91C_SPI_MSTR + AT91C_SPI_MODFDIS ); //Variable addresses
	
		// Setup the 4 SPI channels for SPI0  OBS!! ALL on CSR0  ????????????????????????????????
	// -----------------------------------------------------
	
	// For device 0 ( XC9635XL ) set speed to MCKL/32 and 8 bits, CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, ( SPI_clock_size_Combo << 8 ) | AT91C_SPI_BITS_8 | AT91C_SPI_NCPHA | AT91C_SPI_CSAAT | ( 1 << 24 ), CSR0 );
	// For device 0 ( XC9635XL ) set speed to MCKL/32 and 8 bits, CS active after transfer, mode 0, 1 clk delay b4 CS up
//20090327	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, ( 8 << 8 ) | AT91C_SPI_BITS_8 | AT91C_SPI_NCPHA | AT91C_SPI_CSAAT | ( 1 << 24 ), CSR1 );
	// For device 0 ( XC9635XL ) set speed to MCKL/32 and 8 bits, CS active after transfer, mode 0, 1 clk delay b4 CS up
//20090327	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, ( 8 << 8 ) | AT91C_SPI_BITS_8 | AT91C_SPI_NCPHA | AT91C_SPI_CSAAT | ( 1 << 24 ), CSR2 );
	// For device 0 ( XC9635XL ) set speed to MCKL/32 and 8 bits, CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, ( SPI_clock_size_EEPROM << 8 ) | AT91C_SPI_BITS_8 | AT91C_SPI_NCPHA | AT91C_SPI_CSAAT | ( 1 << 24 ), CSR3 );

	// Issue a last transfer command in order to make the NPCS0 signal rise (inactive)
	pSPI -> SPI_CR = AT91C_SPI_LASTXFER;
};
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SPI_Write_IO_port ( BYTE device, BYTE port, BYTE * data, UINT count )
{
	volatile UINT Data_To_Send, csr, SPIaddress, address, downcount = count, subaddress;
	
	if ( device == COMBO_PORTSPI )
	{
		SPIaddress = ADD_COMBO_MCP23S17;
		address = COMBO_PORTSPI_Write;
		csr = CSR0;
		EnableComboIO;
	}
	else
		return;
	if ( port == A_PORT )
		subaddress = MCP23S17_CMD_OUTPUT_A;
	else if ( port == B_PORT )
		subaddress = MCP23S17_CMD_OUTPUT_B;
	else if ( port == A_CONFIG )
		subaddress = MCP23S17_CMD_OUTPUT_CONFIG_A;
	else if ( port == B_CONFIG )
		subaddress = MCP23S17_CMD_OUTPUT_CONFIG_B;
	
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
	// Cfg SPI MR
	// SPI is in master mode + CS not variable + CS directly connected + 
	// no Fault mode + min DLYBCS
	AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | SPIaddress );
	// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 8 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_8 |
										AT91C_SPI_NCPHA |
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										csr );

	Data_To_Send = address | SPIaddress;  //Opcode
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = subaddress | SPIaddress;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	while ( downcount-- > 1 )
	{
		Data_To_Send = *data++ | SPIaddress;
		AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
		AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
		while (spi_Tdre_Flag == 0);
	};
	Data_To_Send = *data | SPIaddress | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE SPI_Read_IO_port ( BYTE device, BYTE port, BYTE * data, UINT count )
//BYTE	GetADCvalue ( BYTE AdcChannel, AT91PS_SPI AT91C_BASE_SPI0 )
{
	volatile UINT	Data_To_Send, csr, SPIaddress, address, downcount = count, subaddress;
	BYTE						RXvalue;
  UINT						Data_Recieved = 0;
	
	if ( device == COMBO_PORTSPI )
	{
		SPIaddress = ADD_COMBO_MCP23S17;
		address = COMBO_PORTSPI_Read;
		csr = CSR0;
		EnableComboIO;
	};
	if ( port == A_PORT )
		subaddress = MCP23S17_CMD_INPUT_A;
	else if ( port == B_PORT )
		subaddress = MCP23S17_CMD_INPUT_B;

	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414

// Cfg SPI MR
		// SPI is in master mode + CS not variable + CS directly connected + 
		// no Fault mode + min DLYBCS
		AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | SPIaddress );
		// For device 0 ( MCP23S17 ) set speed to MCKL/SPI_clock_size_Combo and 8 bits, 
		// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 8 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_8 | 
										AT91C_SPI_NCPHA |
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										csr );

	Data_To_Send = address | SPIaddress;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	Data_Recieved = AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = subaddress | SPIaddress;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	Data_Recieved = AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	while ( downcount-- > 1 )
	{
		Data_To_Send = 0 | SPIaddress | LASTXFER;
		AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
		while (spi_Tdre_Flag == 0);
		Data_Recieved = WaitUntilReady2RX ( AT91C_BASE_SPI0 );
		*data++ = ( Data_Recieved & 0xFF ) ;
	};
	Data_To_Send = 0 | SPIaddress | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	while (spi_Tdre_Flag == 0);
	Data_Recieved = WaitUntilReady2RX ( AT91C_BASE_SPI0 );
	*data = ( Data_Recieved & 0xFF ) ;

	if ( device == COMBO_PORTSPI )
	{
		EnableADC; //Necessary???
	};

	return RXvalue;	
};	

