//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : Smeter.c
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------

// Include the board file description
#include "Board.h"
	 
#include <string.h> 
#include <inttypes.h>

#include "display.h"
#include "menu.h"
#include "eepromhandler.h"
#include "Smeter.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++

const WORD  cSmeterValues [ 12 ] = { 4600, 4500, 4400, 4250, 
																	4050, 3850, 3650, 3350, 
																	3050, 2600, 2200, 0 };
//const WORD  cSmeterValue =  3350;
	volatile WORD ramSmeterValues	[ 12 ];										
	volatile WORD intermediateSmeterValue;										
	volatile BYTE SMeterStep = 0;
	volatile BYTE SMeterMode = GRAPHIC;
	volatile WORD DVMReference;
//+++++++++++++++++++++++++++++++++++++++++++++++++

void	MakeSmeterCharacters ( void )
{
	char  sign;
	BYTE	PatternAddress;
	BYTE		idx;
	
	// Set address to homemade pattern CGRAM!
	PatternAddress = 0;
	SendCmdToDisplay ( CGRAMSET + PatternAddress, EXTENDEDEXETIME );//39us Using interrupts....
	sign = 0x18; //Double column:  character 0x00
	for ( idx = 0; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x1B;//Two Double columns:  character 0x01
	for ( idx = 0; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
		
	sign = 0x18; //Double column plus Double short column:  character 0x02
	for ( idx = 0; idx < 4; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x1B;
	for ( idx = 4; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
		
	sign = 0x0;//One Double short column:  character 0x03
	for ( idx = 0; idx < 4; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x18;
	for ( idx = 4; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );

	sign = 0x0;//Two Double short column:  character 0x04
	for ( idx = 0; idx < 4; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x1B;
	for ( idx = 4; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
	
	sign = 0x18;//One Double short column:  character 0x05 to mark pos 5
	for ( idx = 0; idx < 7; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x0;
	for ( idx = 7; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
		
	sign = 0x1B;//One Double short column:  character 0x06 to mark pos 5
	for ( idx = 0; idx < 7; idx++ )
		SendCharacterPatternToDisplay ( sign );
	sign = 0x3;
	for ( idx = 7; idx < 8; idx++ )
		SendCharacterPatternToDisplay ( sign );
		
	// Set buffer to DGRAM again
	SendCmdToDisplay ( DGRAMSET, NORMALEXETIME );//39us Using interrupts....

};

//+++++++++++++++++++++++++++++++++++++++++++++++++

//Show Smeter w up to 12 steps (S9+30)
void ShowSMeterValue ( UINT SMeterValue, BYTE mode, BYTE position )
{
	BYTE		idx;
	
	for ( idx = 0; idx <  7; idx++ )
		DisplayBuffer[ idx + position ] = ' ';
		

	if (( mode == GRAPHIC ) || ( mode == POWERMETER ))
	{
			DisplayBuffer [ position ] = 0; 
		if ( SMeterValue > 0 )
			DisplayBuffer [ position ] = 1; 
		if ( SMeterValue > 1 )
			DisplayBuffer [ position + 1 ] = 0; 
		if ( SMeterValue > 2 )
			DisplayBuffer [ position + 1 ] = 1; 
		if ( SMeterValue > 3 )
			DisplayBuffer [ position + 2 ] = 5; // Not full length
		if ( SMeterValue > 4 )
			DisplayBuffer [ position + 2 ] = 6; // Not full length AND a full length
		if ( SMeterValue > 5 )
			DisplayBuffer [ position + 3 ] = 0; 
		if ( SMeterValue > 6 )
			DisplayBuffer [ position + 3 ] = 1; 
		if ( SMeterValue > 7 )
			DisplayBuffer [ position + 4 ] = 0; 
		if ( SMeterValue > 8 )
			DisplayBuffer [ position + 4 ] = 2;//Double column plus Double short column 
			
		if ( SMeterValue > 9 )
			DisplayBuffer [ position + 5 ] = 3;//One Double short column 
		if ( SMeterValue > 10 )
			DisplayBuffer [ position + 5 ] = 4;//Two Double short column
	}
	else
	{
		DisplayBuffer [ position + 1 ] = 'S'; 
		DisplayBuffer [ position + 2 ] = '1'; 
		if ( SMeterValue == 1 )
			DisplayBuffer [ position + 2 ] = '2'; 
		if ( SMeterValue == 2 )
			DisplayBuffer [ position + 2 ] = '3'; 
		if ( SMeterValue == 3 )
			DisplayBuffer [ position + 2 ] = '4'; 
		if ( SMeterValue == 4 )
			DisplayBuffer [ position + 2 ] = '5'; 
		if ( SMeterValue == 5 )
			DisplayBuffer [ position + 2 ] = '6'; 
		if ( SMeterValue == 6 )
			DisplayBuffer [ position + 2 ] = '7'; 
		if ( SMeterValue == 7 )
			DisplayBuffer [ position + 2 ] = '8'; 
		if ( SMeterValue >  7 )
			DisplayBuffer [ position + 2 ] = '9'; 
		if ( SMeterValue >  8 )
		{
			DisplayBuffer [ position + 3 ] = '+'; 
			DisplayBuffer [ position + 5 ] = '0'; 
		};
		if ( SMeterValue == 9 )
			DisplayBuffer [ position + 4 ] = '1'; 
		if ( SMeterValue == 10 )
			DisplayBuffer [ position + 4 ] = '2'; 
		if ( SMeterValue == 11 )
			DisplayBuffer [ position + 4 ] = '3'; 
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

//Show power voltage
void ShowVoltageValue ( UINT SMeterValue,  BYTE position )
{
	BYTE		idx;
	UINT		voltage;
	
	
	for ( idx = 0; idx <  6; idx++ )
		DisplayBuffer[ idx + position ] = ' ';
		
	voltage = SMeterValue * ( ADCREFERENCE * ( AT_VOLT / 100 ));
//	voltage /=  (( VOLTREFERENCE ) * 4096 );
	voltage /=  (( DVMReference ) * 4096 );

	MakeWord2string ( (char*)&DisplayBuffer [ position ], (WORD)voltage, 0 );
//  DisplayBuffer [ position + 3 ] = DisplayBuffer [ position + 2 ]; 
//  DisplayBuffer [ position + 2 ] = ',';
//  DisplayBuffer [ position + 5 ] = 'V';
  DisplayBuffer [ position + 4 ] = DisplayBuffer [ position + 3 ]; 
  DisplayBuffer [ position + 3 ] = ',';
  DisplayBuffer [ position + 6 ] = 'V';
	
};
		
//+++++++++++++++++++++++++++++++++++++++++++++++++

void	SetSMeterCal ( void )
{
	WORD volatile  Value2adjust;
	BYTE	offset;
	
	SMeterStep = 7;
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSetSMeterCal [0], strlen ( cSetSMeterCal ));
//	memcpy ( (char*)&intermediateSmeterValue,
//								(char*)&ramSmeterValues, SMeterStep * 2 );
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
										&cSMeterCalNow [0],
										strlen ( cSMeterCalNow ));
//	Value2adjust = &intermediateSmeterValues [ SMeterStep ];
	offset = strlen ( cSMeterCalNow ) + CHAR_ON_ONE_LINE + 2 ;
	Value2adjust = ramSmeterValues [ SMeterStep ];
	int2stringWsign ( (char*)&DisplayBuffer [ offset ], Value2adjust	);
	DisplayBuffer [ offset + 4 ] = 'm';									
	DisplayBuffer [ offset + 5 ] = 'V';									
	if ( menuPosCol == 1 )
	{
//		if ( SMeterStep < 8 )
		{
//			ShowSMeterValue ( SMeterStep + 1, NUMERIC, 12 );
//			ShowSMeterValue ( SMeterStep + 1, NUMERIC, 20 );
	ControlChanged = CheckEncoderWORD ( (WORD*)&ramSmeterValues [ SMeterStep ], 5000, SMETERVALUESTEPS  );

/*			if (( ControlChanged == SOUND_UP ) &&
				( *Value2adjust < ( SMETERMAXVALUE - SMETERVALUESTEPS )))
			{
				*Value2adjust += SMETERVALUESTEPS;
			};
			if (( ControlChanged == SOUND_DOWN ) &&
				( *Value2adjust > SMETERVALUESTEPS ))
			{
				*Value2adjust -= SMETERVALUESTEPS;
			};
			ControlChanged = 0;
*/	
		};
		if ( KnobGoingActive )
		{
			KnobGoingActive = false;
			AlreadyFound = true;
			menuPosCol++;
		};
	}
	if ( menuPosCol > 1 )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cSaveWith [0], strlen ( cSaveWith ));
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cSkipSaveWith [0], strlen ( cSkipSaveWith ) );
		if ( KnobGoingActive )
		{
			memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
			KnobGoingActive = false;
			AlreadyFound = true;
//			memcpy ( (char*)ramSmeterValues, (char*)&intermediateSmeterValues[0], 24 );
			send2EEPROM ( AddwSmeterValues, (BYTE*)&ramSmeterValues, 24 );
			SMeterStep = 7;
	//		mainmode = RXmode;
			if ( menuPosRow > 1 )
				menuPosRow--;
			menuPosCol = 0;
		};
		return;
	};
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE	GetSMeterValue ( void )
{
	WORD SMeterOut;
	BYTE	idx;
	
	SMeterOut = GetADCvalue ( SMETER_ADCCHANNEL);
	for ( idx = 0; idx < 12; idx++ )
	{
		if ((( ramSmeterValues [ idx ] * 4096 ) / 5000 ) < SMeterOut )
			goto ValueFound;
		
	};
ValueFound:
	return idx; 
};
		
//+++++++++++++++++++++++++++++++++++++++++++++++++

WORD	GetPowerMeterValue ( BYTE channel )
{
	WORD SMeterOut;
	BYTE	idx;
	
	SMeterOut = GetADCvalue ( channel);
	
ValueFound:
	return SMeterOut; 
};
		
//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetupSMetertypeStart ( void )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_ONE_LINE );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSMeterTypStart [0], strlen ( cSMeterTypStart ));
};

//++++++++++++++++++++++++++++++++++++++++++++

void SetupSMetertype ( void )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
  if ( menuPosCol == 0 ) 
  {
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cSMeterTypStart [0], strlen ( cSMeterTypStart ));
	}
	else
	{
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cSMeterTyp[0], strlen ( cSMeterTyp ));
	};
	if ( SMeterMode == GRAPHIC )
		memcpy ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], &cSMeterGraph[0], 13 );
	if ( SMeterMode == NUMERIC )
		memcpy ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], &cSMeterNumeric[0], 13 );
	
	if ( CheckEncoderBYTE ( (BYTE*)&SMeterMode, 1, 1 )) 
	{
		send2EEPROM ( AddSMeterTyp, (BYTE*)&SMeterMode, 1 );
	};
	if ( KnobGoingActive )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
//		mainmode = RXmode;
	};
};

//++++++++++++++++++++++++++++++++++++++++++++

void SetupDVM ( void )
{
	WORD     CalculatingValue;
	
  if ( menuPosCol == 0 ) 
  {
		if (( DVMReference < ( VOLTREFERENCE - 720 )) ||
				( DVMReference > ( VOLTREFERENCE + 720 )))
		{
			DVMReference = VOLTREFERENCE; 
			send2EEPROM ( AddDVMReference, (BYTE*)&DVMReference, 2 );
		};
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cDVMRefStart [0], 
																						strlen ( cDVMRefStart ));
	};
  memset ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], ' ', CHAR_ON_ONE_LINE );
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cDVMRef [0], 
																	strlen ( cDVMRef ));

	CalculatingValue = 2*VOLTREFERENCE - DVMReference;
	ShowVoltageValue ( GetPowerMeterValue ( SMETER_VOLTCHANNEL ), 
										 strlen ( cDVMRef ) + 22 );
  if ( menuPosCol > 0 ) 
  {

		if ( CheckEncoderWORD ( (WORD*)&CalculatingValue, 
														VOLTREFERENCE + 720, 25 ) ) 
		{
			if ( CalculatingValue < ( VOLTREFERENCE - 720 )) 
				CalculatingValue = VOLTREFERENCE - 720; 
			DVMReference = 2*VOLTREFERENCE - CalculatingValue;
		};
	};
	if ( KnobGoingActive )
	{
		send2EEPROM ( AddDVMReference, (BYTE*)&DVMReference, 2 );
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
//		mainmode = RXmode;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};


};

//++++++++++++++++++++++++++++++++++++++++++++


