//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : TWI_ctrl.c
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------


// Include the board file description
#include <string.h>

#include <inttypes.h>
#include "Board.h"
#include "TWI_ctrl.h"


//*=========================================================
//*		WRITE
//*=========================================================
//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Write
//* \brief Send 1 byte to a slave device
//*----------------------------------------------------------------------------

void AT91_TWI_Write_port ( BYTE	port, BYTE data2send )
{
	BYTE TXbuf [ 2 ];
	
	if ( port == 0 ) 
		TXbuf [ 0 ] = PCA9535_CMD_OUTPUT_A;
	else
		TXbuf [ 0 ] = PCA9535_CMD_OUTPUT_B;
	TXbuf [ 1 ] = data2send;
	AT91_TWI_Write ( AT91C_BASE_TWI, ComboPCA9535address_Write, &TXbuf[0], 2 );
};

//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Write
//* \brief Send 2 bytes to a slave device
//*----------------------------------------------------------------------------

void AT91_TWI_Write_2_both_ports ( BYTE data2send0, BYTE data2send1 )
{
	BYTE TXbuf [ 3 ];
	
	TXbuf [ 0 ] = PCA9535_CMD_OUTPUT_A;
	TXbuf [ 1 ] = data2send0;
	TXbuf [ 2 ] = data2send1;
	AT91_TWI_Write ( AT91C_BASE_TWI, ComboPCA9535address_Write, &TXbuf[0], 3 );
};

//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Write
//* \brief Send n bytes to a slave device
//*----------------------------------------------------------------------------
int AT91_TWI_Write(const AT91PS_TWI none, BYTE address, BYTE *data2send, int size )
{
	unsigned int status;
	int						memSize = size;
	BYTE					test;
	
	// Set TWI to 400kHz
//	AT91C_BASE_TWI->TWI_CWGR = (( 3 << 16 ) | ( 0x07 << 8 ) | 0x07 ); // CKDIV | CHDIV | CKDIV
//	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_MSEN;

	// Set the TWI Master Mode Register
	AT91C_BASE_TWI->TWI_MMR = ( address << 15 ) & ~AT91C_TWI_MREAD;	

	AT91C_BASE_TWI->TWI_THR = *(data2send++);  // Subaddress
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_START;

	while ( size-- > 1 )
	{
		// Wait THR Holding register to be empty
		while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXRDY));
	
		// Send first byte
		AT91C_BASE_TWI->TWI_THR = *(data2send++);
	};

	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_STOP;		

	// Wait transfer is finished
   while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXCOMP));
		
	return AT91_TWI_WRITE_OK;
}

//*=========================================================
//*		READ
//*=========================================================
//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Read
//* \brief Read n bytes from a slave device
//*----------------------------------------------------------------------------
int AT91_TWI_Read(const AT91PS_TWI none , int address, int subaddress, BYTE *data, int size)
{
	unsigned int status;
	int						memSize = size;
	
	// Set the TWI Master Mode Register
//	AT91C_BASE_TWI->TWI_MMR = AT91C_EEPROM_I2C_ADDRESS | AT91C_TWI_IADRSZ_2_BYTE | AT91C_TWI_MREAD;	//For EEPROM
	AT91C_BASE_TWI->TWI_MMR = ( address << 15 ) | AT91C_TWI_IADRSZ_1_BYTE | AT91C_TWI_MREAD;	
	
	// Set TWI Internal Address Register
	AT91C_BASE_TWI->TWI_IADR = subaddress;
	
	// Start transfer
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_START;
		
	status = AT91C_BASE_TWI->TWI_SR;
		
	while ( memSize-- > 1 )
	{
		
		// Wait RHR Holding register is full
		while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_RXRDY));

		// Read byte
		*(data++) = AT91C_BASE_TWI->TWI_RHR;
	
	}
	
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_STOP;

	status = AT91C_BASE_TWI->TWI_SR;

	// Wait transfer is finished
    while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXCOMP));

	// Read last byte
	*(data) = AT91C_BASE_TWI->TWI_RHR;
		
	return AT91_TWI_READ_OK;
}


//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Write
//* \brief Send n bytes to a slave device
//*----------------------------------------------------------------------------
int TWI_Write_PCA9535_port ( BYTE device, BYTE port, BYTE *data2send, BYTE size )
{
	unsigned int status;
	int						memSize = size;
	BYTE					test, address, subaddress;
	
	if ( device == COMBO_PORTTWI )
	{
		address = ComboPCA9535address_Write;
	};
	if ( port == A_PORT )
		subaddress = PCA9535_CMD_OUTPUT_A;
	else if ( port == B_PORT )
		subaddress = PCA9535_CMD_OUTPUT_B;
	else if ( port == A_CONFIG )
		subaddress = PCA9535_CMD_OUTPUT_CONFIG_A;
	else if ( port == B_CONFIG )
		subaddress = PCA9535_CMD_OUTPUT_CONFIG_B;

	// Set the TWI Master Mode Register
	AT91C_BASE_TWI->TWI_MMR = ( address << 15 ) & ~AT91C_TWI_MREAD;	

	AT91C_BASE_TWI->TWI_THR = subaddress;  // Subaddress
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_START;

	while ( size-- > 0 )
	{
		// Wait THR Holding register to be empty
		while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXRDY));
	
		// Send first byte
		AT91C_BASE_TWI->TWI_THR = *(data2send++);
	};

	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_STOP;		

	// Wait transfer is finished
   while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXCOMP));
		
	return AT91_TWI_WRITE_OK;
}

//*=========================================================
//*		READ
//*=========================================================
//*----------------------------------------------------------------------------
//* \fn    AT91_TWI_Read
//* \brief Read n bytes from a slave device
//*----------------------------------------------------------------------------
int TWI_Read_PCA9535_port( BYTE device, BYTE port, BYTE *data, BYTE size )
{
	unsigned int status;
	int						memSize = size;
  int          address, subaddress;
	
	if ( device == COMBO_PORTTWI )
	{
		address = ComboPCA9535address_Read;
	};
	if ( port == A_PORT )
		subaddress = PCA9535_CMD_INPUT_A;
	else if ( port == B_PORT )
		subaddress = PCA9535_CMD_INPUT_B;

	// Set the TWI Master Mode Register
	AT91C_BASE_TWI->TWI_MMR = ( address << 15 ) | AT91C_TWI_IADRSZ_1_BYTE | AT91C_TWI_MREAD;	
	
	// Set TWI Internal Address Register
	AT91C_BASE_TWI->TWI_IADR = subaddress;
	
	// Start transfer
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_START;
		
	status = AT91C_BASE_TWI->TWI_SR;
		
	while ( memSize-- > 1 )
	{
		
		// Wait RHR Holding register is full
		while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_RXRDY));

		// Read byte
		*(data++) = AT91C_BASE_TWI->TWI_RHR;
	
	}
	
	AT91C_BASE_TWI->TWI_CR = AT91C_TWI_STOP;

	status = AT91C_BASE_TWI->TWI_SR;

	// Wait transfer is finished
    while (!(AT91C_BASE_TWI->TWI_SR & AT91C_TWI_TXCOMP));

	// Read last byte
	*(data) = AT91C_BASE_TWI->TWI_RHR;
		
	return AT91_TWI_READ_OK;
};
