//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : timerisr.c
// Object              : QROlleSAM7X Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------

#include "Board.h"
#include "buttons.h"
#include "display.h"
#include "menu.h"
#include "HandleDDS.h"

				volatile UINT	ChangeCount 						= 0;// global variable counts interrupts
				volatile UINT	KnobChangeCount 				= 0;// global variable counts interrupts
				volatile UINT	timeSinceLastPulse 			= 0;// global variable counts interrupts
				volatile UINT	EncoderCW_count					= 0;
				volatile UINT	EncoderCCW_count				= 0;
				volatile UINT	ClockwiseRotation				= 0;
				volatile UINT	AntiClockwiseRotation		= 0;
				volatile UINT	timer0count							= 0;
				volatile UINT	timer1count							= 0;
				volatile UINT	timer3count							= 0;
				volatile UINT	Displaycount						= 0;
static volatile BYTE	DotCount          		  = 0;
static volatile BYTE	DahCount          		  = 0;
static volatile BYTE	SpaceCount          		= 0;
static volatile WORD	wBugCountDown      			= 0;
       volatile WORD		wBugCountValue      		= 0;
       volatile WORD		wBreakInDelay      			= 0;
static volatile BYTE	bLongDash								= 0;
static volatile BYTE	bShortDot								= 0;
static volatile BYTE	bIambicActive						= 0;
				volatile BYTE	bIambicMode;
				volatile WORD  LowFqTone								= 0;
				volatile WORD  HighFqTone							= 0;
				volatile WORD  SummerCounter						= 0;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Timer0IrqHandler (void) 
{
         
	volatile AT91PS_TC 		pTC = AT91C_BASE_TC0;		// pointer to timer channel 0 register structure
//	volatile AT91PS_PIO		pPIOA = AT91C_BASE_PIOA;		// pointer to PIO register structure
	unsigned int 			dummy;						// temporary

	dummy = pTC->TC_SR;									// read TC0 Status Register to clear it	
	
	BYTE 		InportNow;
	
  if ( timeSinceLastSave )
    timeSinceLastSave--;
		
  if ( TimeInMainLoop )
    TimeInMainLoop--;
		
  if ( subfunctiontiming )
    subfunctiontiming++;
		
  if ( timing0 )
    timing0--;
		
  if ( DisplayTiming )
    DisplayTiming--;	
		
  if ( ShowVolume )
    ShowVolume--;	
		
  if ( ShowPower )
    ShowPower--;	
		
  if ( ShowVoltage )
    ShowVoltage--;	
		
  if ( ShowBugSpeed )
    ShowBugSpeed--;		
	
  if ( wDisplayBackLiteTime > 1 )
    wDisplayBackLiteTime--;			
	

	if ( wBreakInDelay )
	{
		if ( wBreakInDelay > 4 )
		{
			wBreakInDelay -= 5; //This is a 5 ms timer???
		}
		else
		{
			wBreakInDelay = 0; //Safety...
		};
	}
	else
	{
		ResetBitIOPTT;
	};

	InportNow = 0;
  // Check all switches...
  if ( ButtonActionFlt )
  	InportNow |= 0x01;
  if ( ButtonActionPre )
  	InportNow |= 0x02;
  if ( ButtonActionMd )
  	InportNow |= 0x04;
  if ( ButtonActionCom )
  	InportNow |= 0x08;
  if ( ButtonActionABSplit )
  	InportNow = 0x10;
  if ( ButtonActionBdw )
  	InportNow |= 0x20;
  if ( ButtonActionstep )
  	InportNow |= 0x40;
  if ( ButtonActionBup )
  	InportNow |= 0x80;

	if ( InportNow == LastValidInport )
	{
		ChangeCount = 0;
		if (SameButtonValue < 255 )
			SameButtonValue++;
	}
	else
  {
		LastValidInport = 0;
		SameButtonValue = 0;
		if ( ChangeCount < 255 )
			ChangeCount++;
		if ( ChangeCount > DEBOUNCE )
		{
	    LowFqTone = wButtonSoundLen * 2; // 50 us interrupt for Timer 1!
			LastValidInport = InportNow;
			ChangeCount = 0;
			AlreadyFound = false;
		};
  };
	if ( EncoderSwitch_active )
	{
		if ( KnobChangeCount < KNOBDEBOUNCE )
			KnobChangeCount++;
		if (( KnobChangeCount == KNOBDEBOUNCE ) && ( KnobNowActive == false ))
		{
			KnobGoingActive = true;
			KnobNowActive = true;
			AlreadyFound = false;
		};
	}
	else
	{
		KnobChangeCount = 0;
		KnobNowActive = false;
		KnobGoingActive = false;
	};
	

/*	if  ((pPIOA->PIO_ODSR & aDualLEDred) == aDualLEDred)
		pPIOA->PIO_CODR = aDualLEDred;							// turn LED2 (DS2) on	
	else
		pPIOA->PIO_SODR = aDualLEDred;							// turn LED2 (DS2) off
*/		
		
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Timer1IrqHandler (void) 
{
         
	volatile AT91PS_TC 		pTC = AT91C_BASE_TC1;		// pointer to timer channel 1 register structure
	volatile UINT 			dummy;						// temporary
	volatile BYTE 			BuggLongActive;
	volatile BYTE 			BuggShortActive;

	dummy = pTC->TC_SR;									// read TC1 Status Register to clear it	

	if (	LowFqTone > 0 )
	{
		LowFqTone--;
		if ( SummerCounter > 0 )
		{
			SummerCounter--;
 //     if ( SummerCounter == 0 )//Trig 555
      if ( SummerCounter > ( LENGTH_OF_BUTTON_PERIOD / 2 ))//Trig 555
			{
				EnableSummer;
//        DisableSummer;
			}
			else
			{
        DisableSummer;
			};
		}
		else
		{
      SummerCounter = LENGTH_OF_BUTTON_PERIOD; // 50 us interrupt!
		};
		
	}
	else 
	{
		DisableSummer;
	};

  if ( TimeBetweenChars )
    TimeBetweenChars--;		
 
	if ( timer1count )
		timer1count--;
		
//20081209 ->
	
	if ( BugConnection == 0 ) 
	{
		BuggLongActive = BUG_Conn0_ACTIVE;
		BuggShortActive = BUG_Conn1_ACTIVE;
	}
	else
	{
		BuggLongActive = BUG_Conn1_ACTIVE;
		BuggShortActive = BUG_Conn0_ACTIVE;
	};
  if ( BuggLongActive || BuggShortActive )
		SetPTTifNotExercise;
		

  if ( bIambicMode == StraightKey )
	{
    if ( BuggLongActive || BuggShortActive )
		{
			CloseKey;
			wBreakInDelay = wUsedDelay;
		}
		else
		{
			OpenKey;
		};
		goto NoBugAction;
	};

  if ( wBugCountDown > 0 )
	{
		wBugCountDown--;
		if ( wBugCountDown > 0 )
			goto NoBugAction;
	};
//20081209 <-
	wBugCountDown = wBugCountValue;
//	wBugCountDown = 296;
	 
		if ( DotCount )
		{
			bLongDash			= 0;
			bShortDot			= 1;
			DotCount = 0;
			OpenKey;
			wBreakInDelay = wUsedDelay;
			goto Timer1out;
		};
		if ( DahCount )
		{
			bLongDash			= 1;
			bShortDot			= 0;
			DahCount--;
			if ( !DahCount )
			{
				OpenKey;
				wBreakInDelay = wUsedDelay;
			};
			goto Timer1out;
		};
		if ( bIambicMode )
		{
			if (( BuggLongActive )  && ( BuggShortActive ))
			{
				bIambicActive = 1;
				if ( bShortDot )
				{
					DahCount = 3;
					CloseKey;
					goto Timer1out;
				}
				else
				{
					DotCount = 1;
					CloseKey;
					goto Timer1out;
				};	
			}
			else
			{
				if (( bLongDash ) && 
						( bIambicActive ) &&
						( bIambicMode == IambicModeB ))
				{
					DotCount = 1;
					CloseKey;
					goto Timer1out;
				};
				bIambicActive = 0;
			};
		};
		if ( BuggShortActive )
		{
			DotCount = 1;
			CloseKey;
			goto Timer1out;
		};	
    if ( BuggLongActive )
    {
      DahCount = 3;
      CloseKey;
      goto Timer1out;
    };  
NoBugAction:;
	
Timer1out:;
	

};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Timer2IrqHandler (void) 
{
         
	volatile AT91PS_TC 		pTC = AT91C_BASE_TC2;		// pointer to timer channel 2 register structure
	unsigned int 			dummy;						// temporary

	dummy = pTC->TC_SR;									// read TC1 Status Register to clear it	
  UINT no_decoder_active = NoEncoderActive;
  
	if (( timeSinceLastPulse < ( MAXENCODERCOUNT - 1 )) && no_decoder_active )
	{
		timeSinceLastPulse++;
	};
	if ( no_decoder_active == 0 )
		timeSinceLastPulse = 0;
		
	if ( EncoderDirection == ANTICLOCKWISE )
	{
		ClockwiseRotation = EncoderB_active;
		AntiClockwiseRotation = EncoderA_active;
	}
	else if ( EncoderDirection == CLOCKWISE )
	{
		ClockwiseRotation = EncoderA_active;
		AntiClockwiseRotation = EncoderB_active;
	};
	if (( ClockwiseRotation ) && ( !EncoderCCW_count ))
	{
		if ( EncoderCW_count < ( MAXENCODERCOUNT ))		
			EncoderCW_count++;
		if ( EncoderCW_count == ENCODER_DEBOUNCE )
		{
			ControlChanged = SOUND_UP;
			IncreaseFQ = 1;
		}
	};
	if (( AntiClockwiseRotation ) && ( !EncoderCW_count ))
	{
		if ( EncoderCCW_count <  ( MAXENCODERCOUNT ))		
			EncoderCCW_count++;
		if ( EncoderCCW_count == ENCODER_DEBOUNCE )
		{
			ControlChanged = SOUND_DOWN;
			IncreaseFQ = 2;
		};
	};
	if ( !ClockwiseRotation && !AntiClockwiseRotation )
	{
		EncoderCW_count 	= 0;
		EncoderCCW_count 	= 0;
	};
	
};

/*-----------------------------------------------------------------------------
 * Function Name       : bug_handler
 * Object              : Bug Interrupt Service Routine (PIO A)
 *-----------------------------------------------------------------------------*/
void bug_handler ( void )
{
  /* Read the Interrupt Status register (It acknowledges the Interrupt)  */
  unsigned long pio_isr = AT91C_BASE_PIOA->PIO_ISR;
  unsigned long pio_pdsr = AT91C_BASE_PIOA->PIO_PDSR;


	if (( IsIOEX_set ) || (( param.Modes &  LSBmode ) == CWmode ))
	{
		/* If the Bug has been pushed or released */
	//  20080929		if ( pio_isr & Bug0 )
		{
		  //  20080929			if ( pio_pdsr & Bug0 ) //Check if closed or open...
			{
				OpenKey; //Key is open
				wBreakInDelay = wUsedDelay;
			}
			//  20080929			else
			{	
				CloseKey;
			};
		}
		//  20080929		else if ( pio_isr & Bug1 )
		{
		  //  20080929			if ( pio_pdsr & Bug1 ) //Check if closed or open...
			{
				OpenKey; //Key is open
				wBreakInDelay = wUsedDelay;
			}
			//  20080929			else
			{ 
				CloseKey;
			};
		}
		//  20080929		else
		{ 
		};  /* The else should not be executed since no other PIO are enabled  */
	};
};

